@@ -4,65 +4,56 @@
 #include <string.h>
 #include <time.h>
 #include <getopt.h>
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif /* DMALLOC */
+#ifdef MEMWATCH
+#include "memwatch.h"
+#endif /* MEMWATCH */
 #include "extern.h"
 
-const char *program_name = "authforce";
-const char *author = "Zachary P. Landau";
-const char *version = "0.9.0";
-const char *email = "kapheine@hypa.net";
-const char *description = "an HTTP authentication brute forcer";
+static const char *program_name = "authforce";
+static const char *author = "Zachary P. Landau";
+static const char *version = "0.9.3";
+static const char *email = "kapheine@hypa.net";
+static const char *description = "an HTTP authentication brute forcer";
 
 /* command line configurable variables */
-char datafile_path[60] = "data";			/* location of data files */
-char username_file[80] = "username.lst";	/* file containing usernames */
-char common_passwords_file[80] = "password.lst";	/* file containing common passes */
-char username_concat_file[80]= "concat.lst";/* file containing concats */
-char common_pairs_file[80] = "userpass.lst";/* file containing common pairs */
+char datafile_path[60] = "data";              /* location of data files */
+char username_file[80] = "username.lst";      /* file containing usernames */
+char passwords_file[80] = "password.lst";     /* file with password rules */
+char common_pairs_file[80] = "userpass.lst";  /* file containing common pairs */
 #ifdef USE_DUMMY
-char submit_dummy_file[80] = "dummy.lst";   /* file containing dummy combos */
-#endif USE_DUMMY
-char logfile[80] = "/dev/null";				/* file to log to */
-unsigned int max_users = 0;					/* max number of users to try */
-unsigned int max_connects = 0;				/* max num of connects */
-unsigned int per_user_delay = 0;			/* delay between trying users */
-unsigned int per_password_delay = 0;		/* delay between attempt */
-char user_agent[50];							/* username to pass to browser */
-char url[160];								/* url to bruteforce */
-char proxy[40] = "undef";					/* proxy support */
-int quiet = 0;								/* no output to stdout? */
-int debug = 0;								/* print debug messages? */
-int beep = 0;								/* beep on found? */
+char submit_dummy_file[80] = "dummy.lst";     /* file containing dummy combos */
+#endif /* USE_DUMMY */
+char logfile[80] = "/dev/null";				  /* file to log to */
+unsigned int max_users = 0;					  /* max number of users to try */
+unsigned int max_connects = 0;				  /* max num of connects */
+unsigned int per_user_delay = 0;			  /* delay between trying users */
+unsigned int per_password_delay = 0;		  /* delay between attempt */
+char user_agent[50];						  /* username to pass to browser */
+char url[160];								  /* url to bruteforce */
+char proxy[40] = "undef";					  /* proxy support */
+int quiet = 0;								  /* no output to stdout? */
+int debug = 0;								  /* print debug messages? */
+int beep = 0;								  /* beep on found? */
 char pathlist[120] = DEFAULT_PATH;
 char configlist[120] = DEFAULT_CONFIG;
 
-void *function_list[] = {
-	username_derivations,
-	username_concat,
-	common_passwords,
-	NULL
-};
-
 char *user_pass[2];							/* username, password */
 FILE *log;									/* log file handle */
 unsigned int found = 0;						/* valid auth found */
 unsigned int num_users = 0;					/* cur number of users tried */
 int session_usernumber = 0; 				/* current usernumber */
-int session_function = username_derivations_ident;/* current function */
+int session_function = passwords_ident;     /* current function */
 int session_count = 0;						/* current loop */
 int resume_session = 0;						/* resume old session? */
 int save_session = 0;						/* save session? */
 char session_file[80] = "session.save";		/* file containing session data */
-//unsigned short acs = 0;						/* average connects per second */
+//unsigned short acs = 0;					/* average connects per second */
+char **master_password_list = NULL;         /* master password list */
 
-const char *optstring = "bc:dhl:p:P:qr::s::u:UvV";
-int option_index = 0;
+static const char *optstring = "bc:dhl:p:P:qr::s::u:UvV";
 
 struct option long_options[] = {
 	{ "beep", 0, NULL, 'b' },
-	{ "concat-file", 1, NULL, 0 },
 	{ "debug", 0, NULL, 'd' },
 #ifdef USE_DUMMY
 	{ "dummy-file", 1, NULL, 1 },
@@ -87,36 +78,12 @@ struct option long_options[] = {
 	{ NULL }
 };
 
-static int try_user(char *username) {
-	int i;
-	int (*function)();
-	int result;
-
-	i = session_function;
-
-	while (1) {
-		if (i == common_pairs_ident) 
-			break;
-		function = function_list[i];
-		if (function == NULL || function == (void*)"common_pairs")
-			break;
-		result = function(username);
-		if (result == EXIT_SUCCESS) 
-			return(EXIT_SUCCESS);
-		i++;
-	}
-
-	session_function = 0;
-	return(result);
-}
-
 static void help(char *name) {
 
 	printf("%s %s, %s\n\n", program_name, version, description);
 	printf("usage: %s [OPTION]... URL\n\n", name);
 	printf("options:\n");
 	printf( "\t-b,\t--beep\t\t\tbeep when a match is found\n"
-		 	"\t\t--concat-file=FILE\tfile containing list of concat phrases\n"
 		 	"\t-d,\t--debug\t\t\tprint debug output\n"
 			"\t\t--dummy-file=FILE\tfile containing dummy matches\n"
 			"\t\t\t\t\t[username:password form]\n"
@@ -159,14 +126,13 @@ static void initialize_main(int argc, char *argv[]) {
 
 	snprintf(user_agent, sizeof(user_agent), "%s/%s", program_name, version);
 
-	while ((filename = extract_colon_unit(configlist, &index)) != NULL)
+	while ((filename = extract_colon_unit(configlist, &index)) != NULL) {
 		parse_config(filename);
+		free(filename);
+	}
 
 	while ((optch = getopt_long(argc, argv, optstring, long_options, NULL)) != -1) {
 		switch (optch) {
-			case 0:
-				strncpy(username_concat_file, optarg, sizeof(username_concat_file));
-				break;
 #ifdef USE_DUMMY
 			case 1:
 				strncpy(submit_dummy_file, optarg, sizeof(submit_dummy_file));
@@ -179,7 +145,7 @@ static void initialize_main(int argc, char *argv[]) {
 				per_password_delay = atoi(optarg);
 				break;
 			case 4:
-				strncpy(common_passwords_file, optarg, sizeof(common_passwords_file));
+				strncpy(passwords_file, optarg, sizeof(passwords_file));
 				break;
 			case 5:
 				per_user_delay = atoi(optarg);
@@ -289,9 +255,9 @@ static void shutdown_main(void) {
 }
 
 int main(int argc, char *argv[]) {
-	int i=0;
-	char *username;
+	int i;
 	char **username_list;
+	char *username;
 	int result;
 	char *path = NULL;
 
@@ -301,41 +267,65 @@ int main(int argc, char *argv[]) {
 	initialize_main(argc, argv);
 	initialize_submit();
 
-	path = search_path(username_file, pathlist);
-	username_list = textlist(path);
-
-	if (session_usernumber > num_lines) {
-		fprintf(stderr, "main: session_usernumber greater than num_lines\n");
+	/* cheating way to avoid negative numbers :P */
+	if (abs(session_function) > 1) {
+		fprintf(stderr, "main: session_function greater than 1\n");
 		exit(EXIT_FAILURE);
 	}
 
-	i = session_usernumber;
+	if (session_function == passwords_ident) {
 
-	while ((username = username_list[i]) != NULL) {
-		result = try_user(username);
-		if (result == EXIT_SUCCESS) {
-			if (!quiet) {
-				printf("match [%s:%s]\n", user_pass[0], user_pass[1]);
-				if (beep)
-					printf("\007");
+		path = search_path(username_file, pathlist);
+		username_list = textlist(path);
+		free(path);
+
+		/* session_usernumber starts at 0, but num_lines starts at 1 */
+		if (abs(session_usernumber) > (num_lines+1)) {
+			fprintf(stderr, "main: session_usernumber greater than num_lines\n");
+			exit(EXIT_FAILURE);
+		}
+
+		i = session_usernumber;
+
+		path = search_path(passwords_file, pathlist);
+		master_password_list = textlist(path);
+		free(path);
+
+		/* session_usernumber starts at 0, but num_lines starts at 1 */
+		if (abs(session_count) > (num_lines-1)) {
+			fprintf(stderr, "main: session_count greater than num_lines\n");
+			exit(EXIT_FAILURE);
+		}
+
+		while ((username = username_list[i]) != NULL) {
+			result = process_passwords(username);
+			if (result == EXIT_SUCCESS) {
+				if (!quiet) {
+					printf("match [%s:%s]\n", user_pass[0], user_pass[1]);
+					if (beep)
+						printf("\007");
+				}
+				fprintf(log, "match [%s:%s]\n", user_pass[0], user_pass[1]);
+				found++;
 			}
-			fprintf(log, "match [%s:%s]\n", user_pass[0], user_pass[1]);
-			found++;
+			sleep(per_user_delay);
+			i++;
+			session_usernumber = i;
+			if (max_users == num_users && max_users != 0) {
+				if (!quiet)
+					printf("max users reached at %i\n", num_users);
+				fprintf(log, "max users reached at %i\n", num_users);
+				break;
+			}
+			num_users++;
 		}
-		sleep(per_user_delay);
-		session_usernumber = i;
-		i++;
-		if (max_users == num_users && max_users != 0) {
-			if (!quiet)
-				printf("max users reached at %i\n", num_users);
-			fprintf(log, "max users reached at %i\n", num_users);
-			break;
-		}
-		num_users++;
+		free_list(username_list);
+		free_list(master_password_list);
 	}
 
+
 	/* seperate because common_pairs uses different scheme */
-	common_pairs(num_users);
+	common_pairs();
 
 	if (found > 0) {
 		if (!quiet)
@@ -347,6 +337,12 @@ int main(int argc, char *argv[]) {
 		fprintf(log, "No passwords found.\n");
 	}
 		
+	free(user_pass[0]);
+	free(user_pass[1]);
+#ifdef USE_DUMMY
+	free_list(submit_dummy_list);
+#endif /* USE_DUMMY */
+
 	shutdown_submit();
 	shutdown_main();
 
