@@ -1,6 +1,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
+#ifdef MEMWATCH
+#include "memwatch.h"
+#endif /* MEMWATCH */
 #include "extern.h"
 
 void save_session_handler() {
@@ -14,13 +17,13 @@ void save_session_handler() {
 
 void install_handler() {
 
-	sigset_t usr1;
+	sigset_t sigint;
 
 	if (debug)
-		printf("install_handler: signal SIGUSR installing\n");
+		printf("install_handler: signal SIGINT installing\n");
 
-	signal(SIGUSR1, save_session_handler);
-	sigemptyset(&usr1);
-	sigaddset(&usr1, SIGUSR1);
-	sigprocmask(SIG_UNBLOCK, &usr1, NULL);
+	signal(SIGINT, save_session_handler);
+	sigemptyset(&sigint);
+	sigaddset(&sigint, SIGINT);
+	sigprocmask(SIG_UNBLOCK, &sigint, NULL);
 }
